@@ -8,9 +8,9 @@
 # License, version 2.1 as published by the Free Software Foundation.
 # See the file "COPYING" for the exact licensing terms.
 
-require 'socket'
-require 'thread'
-require 'singleton'
+require "socket"
+require "thread"
+require "singleton"
 
 # = D-Bus main module
 #
@@ -28,7 +28,8 @@ module DBus
 
     # Create a new service with a given _name_ on a given _bus_.
     def initialize(name, bus)
-      @name, @bus = name, bus
+      @name = name
+      @bus = bus
       @root = Node.new("/")
     end
 
@@ -40,20 +41,28 @@ module DBus
     # Perform an introspection on all the objects on the service
     # (starting recursively from the root).
     def introspect
-      if block_given?
-        raise NotImplementedError
-      else
-        rec_introspect(@root, "/")
-      end
+      raise NotImplementedError if block_given?
+
+      rec_introspect(@root, "/")
       self
     end
 
     # Retrieves an object at the given _path_.
     # @return [ProxyObject]
-    def object(path)
-      node = get_node(path, true)
-      if node.object.nil?
-        node.object = ProxyObject.new(@bus, @name, path)
+    def [](path)
+      object(path, api: ApiOptions::A1)
+    end
+
+    # Retrieves an object at the given _path_
+    # whose methods always return an array.
+    # @return [ProxyObject]
+    def object(path, api: ApiOptions::A0)
+      node = get_node(path, _create = true)
+      if node.object.nil? || node.object.api != api
+        node.object = ProxyObject.new(
+          @bus, @name, path,
+          api: api
+        )
       end
       node.object
     end
@@ -68,29 +77,26 @@ module DBus
     # Raises ArgumentError if it is not a DBus::Object.
     # Returns the object, or false if _obj_ was not exported.
     def unexport(obj)
-      raise ArgumentError.new("DBus::Service#unexport() expects a DBus::Object argument") unless obj.kind_of?(DBus::Object)
+      raise ArgumentError, "DBus::Service#unexport() expects a DBus::Object argument" unless obj.is_a?(DBus::Object)
       return false unless obj.path
-      pathSep = obj.path.rindex("/") #last path seperator
-      parent_path = obj.path[1..pathSep-1]
-      node_name = obj.path[pathSep+1..-1]
+      last_path_separator_idx = obj.path.rindex("/")
+      parent_path = obj.path[1..last_path_separator_idx - 1]
+      node_name = obj.path[last_path_separator_idx + 1..-1]
 
       parent_node = get_node(parent_path, false)
       return false unless parent_node
       obj.service = nil
-      parent_node.delete(node_name)
+      parent_node.delete(node_name).object
     end
-	
+
     # Get the object node corresponding to the given _path_. if _create_ is
     # true, the the nodes in the path are created if they do not already exist.
     def get_node(path, create = false)
       n = @root
-      path.sub(/^\//, "").split("/").each do |elem|
-        if not n[elem]
-          if not create
-            return nil
-          else
-            n[elem] = Node.new(elem)
-          end
+      path.sub(%r{^/}, "").split("/").each do |elem|
+        if !(n[elem])
+          return nil if !create
+          n[elem] = Node.new(elem)
         end
         n = n[elem]
       end
@@ -101,7 +107,9 @@ module DBus
     end
 
     #########
+
     private
+
     #########
 
     # Perform a recursive retrospection on the given current _node_
@@ -111,16 +119,15 @@ module DBus
       intfs, subnodes = IntrospectXMLParser.new(xml).parse
       subnodes.each do |nodename|
         subnode = node[nodename] = Node.new(nodename)
-        if path == "/"
-          subpath = "/" + nodename
-        else
-          subpath = path + "/" + nodename
-        end
+        subpath = if path == "/"
+                    "/" + nodename
+                  else
+                    path + "/" + nodename
+                  end
         rec_introspect(subnode, subpath)
       end
-      if intfs.size > 0
-        node.object = ProxyObjectFactory.new(xml, @bus, @name, path).build
-      end
+      return if intfs.empty?
+      node.object = ProxyObjectFactory.new(xml, @bus, @name, path).build
     end
   end
 
@@ -146,18 +153,18 @@ module DBus
 "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
 <node>
 '
-      self.each_pair do |k, v|
+      each_pair do |k, _v|
         xml += "<node name=\"#{k}\" />"
       end
       if @object
-        @object.intfs.each_pair do |k, v|
-          xml += %{<interface name="#{v.name}">\n}
+        @object.intfs.each_pair do |_k, v|
+          xml += %(<interface name="#{v.name}">\n)
           v.methods.each_value { |m| xml += m.to_xml }
           v.signals.each_value { |m| xml += m.to_xml }
-          xml +="</interface>\n"
+          xml += "</interface>\n"
         end
       end
-      xml += '</node>'
+      xml += "</node>"
       xml
     end
 
@@ -170,8 +177,8 @@ module DBus
     # Return instance inspect information, used by Node#inspect.
     def sub_inspect
       s = ""
-      if not @object.nil?
-        s += "%x " % @object.object_id
+      if !@object.nil?
+        s += format("%x ", @object.object_id)
       end
       s + "{" + keys.collect { |k| "#{k} => #{self[k].sub_inspect}" }.join(",") + "}"
     end
@@ -198,9 +205,9 @@ module DBus
     def initialize(path)
       @message_queue = MessageQueue.new(path)
       @unique_name = nil
-      @method_call_replies = Hash.new
-      @method_call_msgs = Hash.new
-      @signal_matchrules = Hash.new
+      @method_call_replies = {}
+      @method_call_msgs = {}
+      @signal_matchrules = {}
       @proxy = nil
       @object_root = Node.new("/")
     end
@@ -209,20 +216,20 @@ module DBus
     # but do not block on the queue.
     # Called by a main loop when something is available in the queue
     def dispatch_message_queue
-      while (msg = @message_queue.pop(:non_block)) # FIXME EOFError
+      while (msg = @message_queue.pop(:non_block)) # FIXME: EOFError
         process(msg)
       end
     end
 
     # Tell a bus to register itself on the glib main loop
     def glibize
-      require 'glib2'
+      require "glib2"
       # Circumvent a ruby-glib bug
-      @channels ||= Array.new
+      @channels ||= []
 
       gio = GLib::IOChannel.new(@message_queue.socket.fileno)
       @channels << gio
-      gio.add_watch(GLib::IOChannel::IN) do |c, ch|
+      gio.add_watch(GLib::IOChannel::IN) do |_c, _ch|
         dispatch_message_queue
         true
       end
@@ -316,8 +323,8 @@ module DBus
     </signal>
   </interface>
 </node>
-'
-# This apostroph is for syntax highlighting editors confused by above xml: "
+'.freeze
+    # This apostroph is for syntax highlighting editors confused by above xml: "
 
     # @api private
     # Send a _message_.
@@ -330,11 +337,8 @@ module DBus
       ret = nil
       if reply_handler.nil?
         send_sync(message) do |rmsg|
-          if rmsg.is_a?(Error)
-            raise rmsg
-          else
-            ret = rmsg.params
-          end
+          raise rmsg if rmsg.is_a?(Error)
+          ret = rmsg.params
         end
       else
         on_return(message) do |rmsg|
@@ -361,8 +365,8 @@ module DBus
         send_sync_or_async(m).first
       else
         send_sync_or_async(m) do |*args|
-          # TODO test async introspection, is it used at all?
-          args.shift            # forget the message, pass only the text
+          # TODO: test async introspection, is it used at all?
+          args.shift # forget the message, pass only the text
           reply_handler.call(*args)
           nil
         end
@@ -379,7 +383,7 @@ module DBus
     # The returned object is a ProxyObject that has methods you can call to
     # issue somme METHOD_CALL messages, and to setup to receive METHOD_RETURN
     def introspect(dest, path)
-      if not block_given?
+      if !block_given?
         # introspect in synchronous !
         data = introspect_data(dest, path)
         pof = DBus::ProxyObjectFactory.new(data, self, dest, path)
@@ -406,23 +410,26 @@ module DBus
       # method calls to be serviced arrive before the reply for RequestName
       # (Ticket#29).
       proxy.RequestName(name, NAME_FLAG_REPLACE_EXISTING) do |rmsg, r|
-        if rmsg.is_a?(Error)  # check and report errors first
-	  raise rmsg
-	elsif r != REQUEST_NAME_REPLY_PRIMARY_OWNER
-          raise NameRequestError
-	end
+        # check and report errors first
+        raise rmsg if rmsg.is_a?(Error)
+        raise NameRequestError unless r == REQUEST_NAME_REPLY_PRIMARY_OWNER
       end
       @service = Service.new(name, self)
       @service
     end
 
     # Set up a ProxyObject for the bus itself, since the bus is introspectable.
+    # @return [ProxyObject] that always returns an array
+    #   ({DBus::ApiOptions#proxy_method_returns_array})
     # Returns the object.
     def proxy
-      if @proxy == nil
+      if @proxy.nil?
         path = "/org/freedesktop/DBus"
         dest = "org.freedesktop.DBus"
-        pof = DBus::ProxyObjectFactory.new(DBUSXMLINTRO, self, dest, path)
+        pof = DBus::ProxyObjectFactory.new(
+          DBUSXMLINTRO, self, dest, path,
+          api: ApiOptions::A0
+        )
         @proxy = pof.build["org.freedesktop.DBus"]
       end
       @proxy
@@ -431,23 +438,23 @@ module DBus
     # @api private
     # Wait for a message to arrive. Return it once it is available.
     def wait_for_message
-      @message_queue.pop                 # FIXME EOFError
+      @message_queue.pop # FIXME: EOFError
     end
 
     # @api private
     # Send a message _m_ on to the bus. This is done synchronously, thus
     # the call will block until a reply message arrives.
     def send_sync(m, &retc) # :yields: reply/return message
-      return if m.nil? #check if somethings wrong
+      return if m.nil? # check if somethings wrong
       @message_queue.push(m)
       @method_call_msgs[m.serial] = m
       @method_call_replies[m.serial] = retc
 
       retm = wait_for_message
-      return if retm.nil? #check if somethings wrong
-      
+      return if retm.nil? # check if somethings wrong
+
       process(retm)
-      while @method_call_replies.has_key? m.serial
+      while @method_call_replies.key? m.serial
         retm = wait_for_message
         process(retm)
       end
@@ -481,23 +488,23 @@ module DBus
 
     def remove_match(mr)
       mrs = mr.to_s
-      unless @signal_matchrules.delete(mrs).nil?
-        # don't remove nonexisting matches.
-        # FIXME if we do try, the Error.MatchRuleNotFound is *not* raised
-        # and instead is reported as "no return code for nil"
-        proxy.RemoveMatch(mrs)
-      end
+      rule_existed = @signal_matchrules.delete(mrs).nil?
+      # don't remove nonexisting matches.
+      return if rule_existed
+      # FIXME: if we do try, the Error.MatchRuleNotFound is *not* raised
+      # and instead is reported as "no return code for nil"
+      proxy.RemoveMatch(mrs)
     end
 
     # @api private
     # Process a message _m_ based on its type.
     def process(m)
-      return if m.nil? #check if somethings wrong
+      return if m.nil? # check if somethings wrong
       case m.message_type
       when Message::ERROR, Message::METHOD_RETURN
-        raise InvalidPacketException if m.reply_serial == nil
+        raise InvalidPacketException if m.reply_serial.nil?
         mcs = @method_call_replies[m.reply_serial]
-        if not mcs
+        if !mcs
           DBus.logger.debug "no return code for mcs: #{mcs.inspect} m: #{m.inspect}"
         else
           if m.message_type == Message::ERROR
@@ -513,25 +520,26 @@ module DBus
           DBus.logger.debug "Got method call on /org/freedesktop/DBus"
         end
         node = @service.get_node(m.path)
-        if not node
+        if !node
           reply = Message.error(m, "org.freedesktop.DBus.Error.UnknownObject",
                                 "Object #{m.path} doesn't exist")
           @message_queue.push(reply)
         # handle introspectable as an exception:
-        elsif m.interface == "org.freedesktop.DBus.Introspectable" and
-            m.member == "Introspect"
+        elsif m.interface == "org.freedesktop.DBus.Introspectable" &&
+              m.member == "Introspect"
           reply = Message.new(Message::METHOD_RETURN).reply_to(m)
           reply.sender = @unique_name
           reply.add_param(Type::STRING, node.to_xml)
           @message_queue.push(reply)
         else
           obj = node.object
-          return if obj.nil?    # FIXME, pushes no reply
+          return if obj.nil? # FIXME, pushes no reply
           obj.dispatch(m) if obj
         end
       when DBus::Message::SIGNAL
         # the signal can match multiple different rules
-        @signal_matchrules.each do |mrs, slot|
+        # clone to allow new signale handlers to be registered
+        @signal_matchrules.dup.each do |mrs, slot|
           if DBus::MatchRule.new.from_s(mrs).match(m)
             slot.call(m)
           end
@@ -550,7 +558,7 @@ module DBus
       # anything
       Service.new(name, self)
     end
-    alias :[] :service
+    alias [] service
 
     # @api private
     # Emit a signal event for the given _service_, object _obj_, interface
@@ -587,7 +595,6 @@ module DBus
     end
   end # class Connection
 
-
   # = D-Bus session bus class
   #
   # The session bus is a session specific bus (mostly for desktop use).
@@ -597,25 +604,32 @@ module DBus
   class ASessionBus < Connection
     # Get the the default session bus.
     def initialize
-      super(ENV["DBUS_SESSION_BUS_ADDRESS"] || address_from_file || "launchd:env=DBUS_LAUNCHD_SESSION_BUS_SOCKET")
+      super(self.class.session_bus_address)
       send_hello
     end
 
-    def address_from_file
+    def self.session_bus_address
+      ENV["DBUS_SESSION_BUS_ADDRESS"] ||
+        address_from_file ||
+        "launchd:env=DBUS_LAUNCHD_SESSION_BUS_SOCKET"
+    end
+
+    def self.address_from_file
       # systemd uses /etc/machine-id
       # traditional dbus uses /var/lib/dbus/machine-id
-      machine_id_path = Dir['{/etc,/var/lib/dbus}/machine-id'].first
+      machine_id_path = Dir["{/etc,/var/lib/dbus,/var/db/dbus}/machine-id"].first
       return nil unless machine_id_path
       machine_id = File.read(machine_id_path).chomp
 
       display = ENV["DISPLAY"][/:(\d+)\.?/, 1]
 
       bus_file_path = File.join(ENV["HOME"], "/.dbus/session-bus/#{machine_id}-#{display}")
-      return nil unless File.exists?(bus_file_path)
+      return nil unless File.exist?(bus_file_path)
 
       File.open(bus_file_path).each_line do |line|
         if line =~ /^DBUS_SESSION_BUS_ADDRESS=(.*)/
-          return $1
+          address = Regexp.last_match(1)
+          return address[/\A'(.*)'\z/, 1] || address[/\A"(.*)"\z/, 1] || address
         end
       end
     end
@@ -626,7 +640,6 @@ module DBus
     include Singleton
   end
 
-
   # = D-Bus system bus class
   #
   # The system bus is a system-wide bus mostly used for global or
@@ -641,22 +654,21 @@ module DBus
       send_hello
     end
   end
-  
+
   # = D-Bus remote (TCP) bus class
   #
-  # This class may be used when connecting to remote (listening on a TCP socket) 
+  # This class may be used when connecting to remote (listening on a TCP socket)
   # busses. You can also use it to connect to other non-standard path busses.
-  # 
+  #
   # The specified socket_name should look like this:
   # (for TCP)         tcp:host=127.0.0.1,port=2687
   # (for Unix-socket) unix:path=/tmp/my_funky_bus_socket
-  # 
-  # you'll need to take care about authentification then, more info here: 
+  #
+  # you'll need to take care about authentification then, more info here:
   # http://github.com/pangdudu/ruby-dbus/blob/master/README.rdoc
   class RemoteBus < Connection
-
     # Get the remote bus.
-    def initialize socket_name
+    def initialize(socket_name)
       super(socket_name)
       send_hello
     end
@@ -669,13 +681,13 @@ module DBus
 
   # Shortcut for the {SystemBus} instance
   # @return [Connection]
-  def DBus.system_bus
+  def self.system_bus
     SystemBus.instance
   end
 
   # Shortcut for the {SessionBus} instance
   # @return [Connection]
-  def DBus.session_bus
+  def self.session_bus
     SessionBus.instance
   end
 
@@ -687,7 +699,7 @@ module DBus
   class Main
     # Create a new main event loop.
     def initialize
-      @buses = Hash.new
+      @buses = {}
       @quitting = false
     end
 
@@ -706,11 +718,11 @@ module DBus
       # before blocking, empty the buffers
       # https://bugzilla.novell.com/show_bug.cgi?id=537401
       @buses.each_value do |b|
-        while m = b.message_queue.message_from_buffer_nonblock
+        while (m = b.message_queue.message_from_buffer_nonblock)
           b.process(m)
         end
       end
-      while not @quitting and not @buses.empty?
+      while !@quitting && !@buses.empty?
         ready = IO.select(@buses.keys, [], [], 5) # timeout 5 seconds
         next unless ready # timeout exceeds so continue unless quitting
         ready.first.each do |socket|
@@ -721,7 +733,7 @@ module DBus
             @buses.delete socket # this bus died
             next
           end
-          while m = b.message_queue.message_from_buffer_nonblock
+          while (m = b.message_queue.message_from_buffer_nonblock)
             b.process(m)
           end
         end
